"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var MemoryFS = require("memory-fs");
var path = require("path");
var webpack = require("webpack");
var webpackDevMiddleware = require("webpack-dev-middleware");
var webpackHotMiddleware = require("webpack-hot-middleware");
exports.HMR_ENTRY = "webpack-hot-middleware/client";
exports.getHmrPluginsByVersion = function () {
    var version = require("webpack/package.json").version;
    var majarVersion = String(version).split(".")[0];
    switch (majarVersion) {
        case "1":
            throw new Error("not support webpack@1");
        case "2":
        default:
            return [
                webpack.HotModuleReplacementPlugin,
            ];
    }
};
var concatHMREntry = function (entry) { return [exports.HMR_ENTRY].concat(entry); };
var isOneOfPlugins = function (PluginList, plugin) {
    return _.reduce(PluginList, function (result, Plugin) { return (result || (plugin instanceof Plugin)); }, false);
};
exports.patchEntryWithHMR = function (entry) {
    if (_.isObject(entry)) {
        return _.mapValues(entry, concatHMREntry);
    }
    return concatHMREntry(entry);
};
exports.patchPlugins = function (plugins) {
    var hmrPlugins = exports.getHmrPluginsByVersion();
    var cleanedPlugins = _.dropWhile(plugins, function (plugin) { return isOneOfPlugins(hmrPlugins, plugin); });
    return _.concat(cleanedPlugins, _.map(hmrPlugins, function (Plugin) { return new Plugin(); }));
};
exports.patchWebConfigWithHMR = function (webpackConfig) { return (__assign({}, webpackConfig, { entry: exports.patchEntryWithHMR(webpackConfig.entry), plugins: exports.patchPlugins(webpackConfig.plugins) })); };
exports.createMiddlewaresForWebpack = function (webpackConfig, index, hot) {
    if (hot === void 0) { hot = false; }
    var patchedWebpackConfig = hot
        ? exports.patchWebConfigWithHMR(webpackConfig)
        : webpackConfig;
    var bundler = webpack(patchedWebpackConfig);
    var fs = new MemoryFS();
    bundler.outputFileSystem = fs;
    var devMiddleware = webpackDevMiddleware(bundler, {
        publicPath: patchedWebpackConfig.output.publicPath,
        stats: patchedWebpackConfig.stats || {
            colors: true,
            reasons: true,
            hash: false,
            version: false,
            timings: true,
            chunks: false,
            chunkModules: false,
            cached: false,
        },
    });
    var devServerMiddlewares = [
        devMiddleware,
        (function (req, res, next) {
            if (req.method === "GET" && req.url === "/") {
                devMiddleware.waitUntilValid(function () {
                    var indexFile = path.join(webpackConfig.output.path, index);
                    res.end(fs.readFileSync(indexFile));
                });
            }
            else {
                next();
            }
        }),
    ];
    if (hot) {
        return devServerMiddlewares.concat([
            webpackHotMiddleware(bundler),
        ]);
    }
    return devServerMiddlewares;
};
