"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var browserSync = require("browser-sync");
var compress = require("compression");
var connectHistoryApiFallback = require("connect-history-api-fallback");
var interpret = require("interpret");
var _ = require("lodash");
var path = require("path");
var utils_1 = require("./utils");
var Webpack_1 = require("./Webpack");
function registerCompiler(moduleDescriptor) {
    if (moduleDescriptor) {
        if (typeof moduleDescriptor === "string") {
            require(moduleDescriptor);
        }
        else if (!Array.isArray(moduleDescriptor)) {
            moduleDescriptor.register(require(moduleDescriptor.module));
        }
        else {
            for (var _i = 0, moduleDescriptor_1 = moduleDescriptor; _i < moduleDescriptor_1.length; _i++) {
                var moduleName = moduleDescriptor_1[_i];
                if (moduleName) {
                    try {
                        registerCompiler(moduleName);
                        break;
                    }
                    catch (e) {
                        // do nothing
                    }
                }
            }
        }
    }
}
var createBrowserSyncOptions = function (options) {
    var webpackConfigFile = path.join(process.cwd(), options.config);
    var webpackConfigFileExt = path.extname(webpackConfigFile);
    registerCompiler(interpret.extensions[webpackConfigFileExt]);
    var webpackConfig = require(webpackConfigFile);
    var middlewares = [];
    if (options.historyApiFallback) {
        middlewares = middlewares.concat(connectHistoryApiFallback({
            index: "/",
        }));
    }
    if (options.webpack) {
        middlewares = middlewares.concat(Webpack_1.createMiddlewaresForWebpack(webpackConfig, options.index, options.hot));
    }
    if (options.compress) {
        middlewares = middlewares.concat(compress({ level: 6 }));
    }
    if (options.proxy) {
        return __assign({}, (_.get(webpackConfig, "devServer.browserSync")), { proxy: {
                target: options.proxy,
                middleware: middlewares,
            } });
    }
    return __assign({}, (_.get(webpackConfig, "devServer.browserSync")), { server: {
            baseDir: utils_1.getBaseDir(webpackConfig.output.path, webpackConfig.output.publicPath),
            index: path.join(webpackConfig.output.publicPath || "/", options.index),
            middleware: middlewares,
        }, notify: false });
};
exports.create = function (opts) { return browserSync(createBrowserSyncOptions(opts)); };
